@@ -617,10 +617,10 @@ def season_to_month_tuple(seasons: Sequence[str]) -> tuple[tuple[int, ...], ...]
     ((12, 1, 2, 3), (9, 10, 11, 12))
     """
     initials = "JFMAMJJASOND"
-    starts = dict(
-        ("".join(s), i + 1)
+    starts = {
+        "".join(s): i + 1
         for s, i in zip(sliding_window(2, initials + "J"), range(12), strict=True)
-    )
+    }
     result: list[tuple[int, ...]] = []
     for i, season in enumerate(seasons):
         if len(season) == 1:
@@ -701,25 +701,23 @@ def find_independent_seasons(seasons: Sequence[str]) -> Sequence[SeasonsGroup]:
     grouped = defaultdict(list)
     codes = defaultdict(list)
     seen: set[tuple[int, ...]] = set()
-    idx = 0
     # This is quadratic, but the number of seasons is at most 12
     for i, current in enumerate(season_inds):
         # Start with a group
         if current not in seen:
-            grouped[idx].append(current)
-            codes[idx].append(i)
+            grouped[i].append(current)
+            codes[i].append(i)
             seen.add(current)
 
         # Loop through remaining groups, and look for overlaps
         for j, second in enumerate(season_inds[i:]):
-            if not (set(chain(*grouped[idx])) & set(second)) and second not in seen:
-                grouped[idx].append(second)
-                codes[idx].append(j + i)
+            if not (set(chain(*grouped[i])) & set(second)) and second not in seen:
+                grouped[i].append(second)
+                codes[i].append(j + i)
                 seen.add(second)
         if len(seen) == len(seasons):
             break
-        # found all non-overlapping groups for this row, increment and start over
-        idx += 1
+        # found all non-overlapping groups for this row start over
 
     grouped_ints = tuple(tuple(idx) for idx in grouped.values() if idx)
     return [
