@@ -1,12 +1,8 @@
 from __future__ import annotations
 
 import sys
-from importlib.metadata import EntryPoint
-
-if sys.version_info >= (3, 10):
-    from importlib.metadata import EntryPoints
-else:
-    EntryPoints = list[EntryPoint]
+from importlib.metadata import EntryPoint, EntryPoints
+from itertools import starmap
 from unittest import mock
 
 import pytest
@@ -26,22 +22,22 @@ importlib_metadata_mock = "importlib.metadata"
 
 
 class DummyBackendEntrypointArgs(common.BackendEntrypoint):
-    def open_dataset(filename_or_obj, *args):
+    def open_dataset(filename_or_obj, *args):  # type: ignore[override]
         pass
 
 
 class DummyBackendEntrypointKwargs(common.BackendEntrypoint):
-    def open_dataset(filename_or_obj, **kwargs):
+    def open_dataset(filename_or_obj, **kwargs):  # type: ignore[override]
         pass
 
 
 class DummyBackendEntrypoint1(common.BackendEntrypoint):
-    def open_dataset(self, filename_or_obj, *, decoder):
+    def open_dataset(self, filename_or_obj, *, decoder):  # type: ignore[override]
         pass
 
 
 class DummyBackendEntrypoint2(common.BackendEntrypoint):
-    def open_dataset(self, filename_or_obj, *, decoder):
+    def open_dataset(self, filename_or_obj, *, decoder):  # type: ignore[override]
         pass
 
 
@@ -53,7 +49,7 @@ def dummy_duplicated_entrypoints():
         ["engine2", "xarray.tests.test_plugins:backend_1", "xarray.backends"],
         ["engine2", "xarray.tests.test_plugins:backend_2", "xarray.backends"],
     ]
-    eps = [EntryPoint(name, value, group) for name, value, group in specs]
+    eps = list(starmap(EntryPoint, specs))
     return eps
 
 
@@ -96,7 +92,7 @@ def test_backends_dict_from_pkg() -> None:
         ["engine1", "xarray.tests.test_plugins:backend_1", "xarray.backends"],
         ["engine2", "xarray.tests.test_plugins:backend_2", "xarray.backends"],
     ]
-    entrypoints = [EntryPoint(name, value, group) for name, value, group in specs]
+    entrypoints = list(starmap(EntryPoint, specs))
     engines = plugins.backends_dict_from_pkg(entrypoints)
     assert len(engines) == 2
     assert engines.keys() == {"engine1", "engine2"}
@@ -258,9 +254,9 @@ def test_lazy_import() -> None:
                 if pkg.startswith(mod):
                     is_imported.add(mod)
                     break
-        assert (
-            len(is_imported) == 0
-        ), f"{is_imported} have been imported but should be lazy"
+        assert len(is_imported) == 0, (
+            f"{is_imported} have been imported but should be lazy"
+        )
 
     finally:
         # restore original
@@ -288,10 +284,7 @@ def test_refresh_engines() -> None:
     EntryPointMock1.name = "test1"
     EntryPointMock1.load.return_value = DummyBackendEntrypoint1
 
-    if sys.version_info >= (3, 10):
-        return_value = EntryPoints([EntryPointMock1])
-    else:
-        return_value = {"xarray.backends": [EntryPointMock1]}
+    return_value = EntryPoints([EntryPointMock1])
 
     with mock.patch("xarray.backends.plugins.entry_points", return_value=return_value):
         list_engines.cache_clear()
@@ -303,10 +296,7 @@ def test_refresh_engines() -> None:
     EntryPointMock2.name = "test2"
     EntryPointMock2.load.return_value = DummyBackendEntrypoint2
 
-    if sys.version_info >= (3, 10):
-        return_value2 = EntryPoints([EntryPointMock2])
-    else:
-        return_value2 = {"xarray.backends": [EntryPointMock2]}
+    return_value2 = EntryPoints([EntryPointMock2])
 
     with mock.patch("xarray.backends.plugins.entry_points", return_value=return_value2):
         refresh_engines()
